"""Utilities for dealing with images"""

import numpy as np


def guess_dpi(image_array):
    h, *_ = image_array.shape
    resolutions = np.array([1200, 600, 400, 300, 200, 150, 120, 100, 75, 60, 50, 40])
    return resolutions[np.argmin(abs(resolutions - 25.4 * h / 297))]


def get_box(image_array, box, padding=0.3):
    """Extract a subblock from an array corresponding to a scanned A4 page.

    Parameters:
    -----------
    image_array : 2D or 3D array
        The image source.
    box : 4 floats (top, bottom, left, right)
        Coordinates of the bounding box in inches. By due to differing
        traditions, box coordinates are counted from the bottom left of the
        image, while image array coordinates are from the top left.
    padding : float
        Padding around box borders in inches.
    """
    # TODO: use points as base unit
    h, w, *_ = image_array.shape
    dpi = guess_dpi(image_array)
    box = np.array(box)
    box += (-padding, padding, -padding, padding)
    box = (np.array(box) * dpi).astype(int)
    # Here we are not returning the lowest pixel of the image because otherwise
    # the numpy slicing is not correct.
    top, bottom = max(0, min(box[0], h)), max(1, min(box[1], h))
    left, right = max(0, min(box[2], w)), max(1, min(box[3], w))
    return image_array[top:bottom, left:right]


def box_is_filled(image_array, box_coords, padding=0.3, threshold=150, pixels=False):
    """
    Determines if a box is filled

    Parameters:
    -----------
    image_array : 2D or 3D array
        The image source.
    box_coords : 4 floats (top, bottom, left, right)
        Coordinates of the bounding box in inches or pixels. By due to differing
        traditions, box coordinates are counted from the bottom left of the
        image, while image array coordinates are from the top left.
    padding : float
        Padding around box borders in inches.
    threshold : int
        Optional threshold value to determine minimal 'darkness'
        to consider a box to be filled in
    pixels : boolean
        Whether the box coordinates are entered as pixels instead of inches.
    """

    # Divide by DPI if pixel coordinates are used
    if pixels:
        box_coords /= guess_dpi(image_array)

    box_img = get_box(image_array, box_coords, padding)

    # Check if the coordinates are outside of the image
    if box_img.size == 0:
        raise RuntimeError("Box coordinates are outside of image")

    avg = np.average(box_img)

    return avg < threshold
